/**************************************************************
* Copyright  Comune di Prato - Sistema Informativo
* Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
* File:       SITExtStore.java
* Author:     Alessandro Radaelli
* Version:    1.0.0
* CreationDate:   Settembre 2010
* ModifyDate:     
* 
*             
***************************************************************/
package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.Arrays;
import java.util.List;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;

/**
 * Classe che contiene i metodi statici per ottenere le rappresentazioni JSON corrispondenti a store di ExtJs
 * 
 * @author Alessandro Radaelli
 *
 */
public class SITExtStore {
    
    /**
     * Crea i metadati che saranno allegati allo store Ext
     * 
     * @param obj oggetti cge sar contenuto nello store del quale vengono creati i metadati
     * @param jsonConfig Pu essere null, se non lo  contiene la configurazione che sar usata nei metodi di creazione degli oggetti JSONObject
     * @return Ritorna i metadati creato
     */
    static public JSONObject extStoreMetadata(Object obj, JsonConfig jsonConfig ){
        // Metadata
        JSONObject metaData = new JSONObject();
        JSONArray metadataFields = new JSONArray();
        metaData.put("root", "rows");
        metaData.put("totalProperty", "total");            
        metaData.put("successProperty", "success");            
               
        if (obj!=null) {
            JSONObject record = (jsonConfig!=null) ? JSONObject.fromObject(obj, jsonConfig) : JSONObject.fromObject(obj);
       
            for(Object key: record.keySet()) {
                JSONObject buff = new JSONObject();
                buff.put("name", key.toString());
                metadataFields.add(buff);
                // Se array oltre ad aggiungere l'oggetto stesso lo appiattisce anche (utilizzato per Lista di suggest)
                if (JSONArray.class.isAssignableFrom(record.get(key).getClass())) {
                    JSONArray arr = (JSONArray) record.get(key);
                    for (int i=0; i<arr.size(); i++) {
                        buff.put("name", key.toString()+i);
                        buff.put("mapping", key.toString()+"[" + i + "]");
                        metadataFields.add(buff);
                    }
                } else {   
                    // Se oggetto composto appiattisce oltre ad aggiungere l'oggetto stesso lo appiattisce anche(come per additionalFields)
                    if (JSONObject.class.isAssignableFrom(record.get(key).getClass())) {
                          JSONObject arr = (JSONObject) record.get(key);
                          for (Object k: arr.keySet()) {
                              buff.put("name", key.toString()+k.toString());
                              buff.put("mapping", key.toString()+"."+k.toString());
                              metadataFields.add(buff);
                          }
                    } 
                }
            }
            metaData.put("fields", metadataFields);
        } else {
            // Se non ci sono oggetti mette un solo campo descrizione altrimenti lato Ext va in errore con success=false
            JSONObject buff = new JSONObject();
            buff.put("name", "Description");
            metadataFields.add(buff);
            metaData.put("fields", metadataFields);
        }
        //metaData.put("ale", 3);
        // recuperabile in store.reader.meta.ale
    
        return metaData;
        
    }
    
    /**
     * Crea lo store Ext che contiene i metadati e le geometrie passate
     * 
     * @param metaData Metadati da allegare allo store
     * @param arr Array di geometrie
     * @param jsonConfig Pu essere null, se non lo  contiene la configurazione che sar usata nei metodi di creazione degli oggetti JSONObject
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato
     */
    static public JSONObject extStore(JSONObject metaData, JSGeometryArrayList<? extends JSGeometry> arr, JsonConfig jsonConfig  ) {
        JSONObject res = new JSONObject();
        
        res.put("success", true);
        res.put("metaData", metaData);
        res.put("total", arr.getGeometries().size());
        res.put("rows", arr.getGeometries());
        
        return (jsonConfig!=null) ?  JSONObject.fromObject(res,jsonConfig) :  JSONObject.fromObject(res);
        
    }
    
    /**
     * Crea lo store Ext che contiene i metadati e gli oggetti passati
     * 
     * @param metaData Metadati da allegare allo store
     * @param arr Oggetti da includere 
     * @param jsonConfig Pu essere null, se non lo  contiene la configurazione che sar usata nei metodi di creazione degli oggetti JSONObject
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato 
     */
    static public JSONObject extStore(JSONObject metaData, JSONArray arr, JsonConfig jsonConfig  ) {
        JSONObject res = new JSONObject();
        
        res.put("success", true);
        res.put("metaData", metaData);
        res.put("total", arr.size());
        res.put("rows", arr);
        
        return (jsonConfig!=null) ?  JSONObject.fromObject(res,jsonConfig) :  JSONObject.fromObject(res);
        
    }
    
    /**
     * Crea lo store Ext che contiene un singolo record con l'oggetto passato (non vengono creati pi record contenenti i singoli oggetti contenuti nella lista ma un unico record contenente la lista). I metadati vengono calcolati automaticamente
     * 
     * @param obj Oggetto da includere nello store
     * @param jsonConfig Ritorna l'oggetto JSONObject corrispondente allo store creato
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato
     */
    static public JSONObject extStoreSingleRecord(Object obj, JsonConfig jsonConfig  ){
    
        JSONObject metaData = extStoreMetadata(obj, jsonConfig);
        JSONArray arr = new JSONArray();
        arr.add(obj);
        
        return extStore(metaData, arr, jsonConfig);
        
    }
    
    /**
     * Crea lo store Ext che contiene la lista di oggetti passati . I metadati vengono calcolati automaticamente
     * @param obj Lista di oggetti da includere nello store
     * @param jsonConfig Ritorna l'oggetto JSONObject corrispondente allo store creato
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato
     */
    static public JSONObject extStore(JSONArray obj, JsonConfig jsonConfig  ){
        
        //JSONObject metaData = extStoreMetadata(obj, jsonConfig);
        JSONObject metaData = (obj.size()>0) ? extStoreMetadata(obj.get(0), jsonConfig) : extStoreMetadata(null, jsonConfig)  ;
        JSONArray arr = new JSONArray();
        arr.addAll(obj);
        
        return extStore(metaData, arr, jsonConfig);
        
    }
    
    /**
     * Crea uno store Ext contenente un record per ogni geometria della lista. I metadati vengono calcolati automaticamente
     *   
     * @param obj Array di geometrie da includere nello store
     * @param jsonConfig Ritorna l'oggetto JSONObject corrispondente allo store creato
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato
     */
    static public JSONObject extStoreFromJSGeometryArrayList(JSGeometryArrayList<? extends JSGeometry> obj, JsonConfig jsonConfig  ){
        
        JSONObject metaData = (obj.getGeometries().size()>0) ? extStoreMetadata(obj.getGeometries().get(0), jsonConfig) : extStoreMetadata(null, jsonConfig)  ;
        metaData.put("boundingBox", obj.getBoundingbox());
        
        return extStore(metaData, obj, jsonConfig);
        
    }
    
    /**
     * Crea uno store Ext a partire da un {@see SITPaginatedResult}.
     * 
     * @param pagRes Risultato paginato della ricerca
     * @param withGeometry Flag che indica se includere o meno la geometria
     * @param toSRID SRID al quale convertire
     * @param jsonConfig oggetto JSONObject per la configurazione dello store  creato  
     * @param logger Logger
     * @return
     * @throws SITException
     */
   static public JSONObject extStoreFromSITPaginatedResul(SITPaginatedResult pagRes, boolean withGeometry, String toSRID, JsonConfig jsonConfig, LogInterface logger  ) throws SITException{
    
	   try {
		   	JSGeometryArrayList<JSGeometry> list = JSGeometryArrayList.toJSGeometryArrayList(pagRes.getResult(), toSRID, logger, withGeometry);
		   
	        JSONObject metaData = (list.getGeometries().size()>0) ? extStoreMetadata(list.getGeometries().get(0), jsonConfig) : extStoreMetadata(null, jsonConfig)  ;
	        metaData.put("boundingBox", list.getBoundingbox());
	        metaData.put("start", pagRes.getStartIndex());
	        metaData.put("limit", pagRes.getMaxFeatures());
	        
	        JSONObject store = extStore(metaData, list, jsonConfig);
	        store.put("total", pagRes.getTotalCount());
	        return store;
	        
	   }catch (SITException e) {
		   logger.warn("Errore durante l'esecuzione del metodo extStoreFromSITPaginatedResul. ", e);
		   throw e;
	   }
       
   }
    
    /**
     * Crea uno store Ext a partire da una stringa. I metadati vengono calcolati automaticamente 
     * 
     * @param str Stringa di descrizioni separate da carriage return (\n)
     * @return Ritorna l'oggetto JSONObject corrispondente allo store creato
     */
    static public JSONObject extStoreFromString(String str){
        
        JSONObject obj = new JSONObject();
        obj.put("Descrizione", "");
        
        JSONObject metaData = extStoreMetadata(obj, null);
        List<String> strsplit = Arrays.asList(str.split("\n"));
        JSONArray dati = new JSONArray();
        
        for (String s: strsplit) {
            JSONObject buff = new JSONObject();
            buff.put("Descrizione", s);
            dati.add(buff);
        }
        
        return extStore(metaData, dati, null);
        
    }
    
}
