/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit.runtime.util;

import it.prato.comune.sit.SITParseException;

import java.text.MessageFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * Classe che implementa la Version per il SIT. <br />
 * Per una versione SIT  prevista anche la gestione degli stati di pre-release e la loro revisione:
 * </p>
 * <ul>
 * <li> 
 *  <b>Alpha (A)</b> : fase embrionale non adatta per il rilascio (distribuita ai soli sviluppatori);
 * </li>
 * <li>
 *  <b>Beta (B)</b> : pi matura rispetto allo fase Alpha, ma non ancora pronta per il rilascio (distribuita ad un gruppo pi largo di tester e sviluppatori)
 * </li>
 * <li>
 *  <b>Release Candidate (RC)</b> : in questa fase la versione  praticamente pronta per il rilascio ed attende il miglior debug possibile dai test eseguiti da un gruppo ampio di utenti.
 * </li>
 * </ul>
 * <p>
 * Per ogni stato di pre-rilascio  previsto un numero di revisione man mano che bugs vengono o modifiche vengono attuate. 
 * </p>
 * <p>
 * Vediamo alcuni esempi di versioni accettate :
 * </p>
 * <code>
 * 1.2.0A0 &nbsp;= versione 1.2.0 in stato rilascio Alpha;
 * <br />
 * 1.2.0B2 &nbsp;= versione 1.2.0 in stato rilascio Beta seconda revisione;
 * <br />
 * 1.2.0RC1 = versione 1.2.0 in stato rilascio Release Candidate prima revisione;
 * </ul>
 * </code>
 * </p>
 * <br />
 * 
 * @author bf44
 *
 */
public class SITVersion implements Version {
    
    /**
     * Enumerazione degli stati di rilascio di una versione e dei relativi codici.
     * @author bf44
     *
     */
    public static enum ReleaseStatus {        
        ALPHA(0,"A","alpha"),        
        BETA(1,"B","beta"),
        RELEASE_CANDIDATE(2,"RC","rc"),
        LIVE_RELEASE(3,"",""),
        SNAPSHOT(4,"SNAPSHOT","SNAPSHOT");
        
        ReleaseStatus(int step, String codice, String descrizione){
           this.step = step;
           this.codice = codice;
           this.descrizione = descrizione;
        }
        
        private final int step;                  
        private final String codice;
        private final String descrizione;
        
        public int getStep() { return step; }
        public String getCodice() { return codice; }
        public String getDescrizione() { return descrizione; }
        
        public static ReleaseStatus decode(String codice){
            for(ReleaseStatus og: ReleaseStatus.values()){
               if(og.getCodice().equals(codice))
                  return og;
            }
            return null;
        }
    }
    
    private int major;
    private int minor;
    private int revision;
    private ReleaseStatus releaseStatus = ReleaseStatus.LIVE_RELEASE;
    private Integer releaseStatusRevision;
            
    /**
     * Costruttore che non accetta lo stato del rilascio. In questo caso lo stato del rilascio viene impostato
     * a quello di default cio a LIVE RELEASE ovvero la versione stabile rilasciata.
     * @param major
     * @param minor
     * @param revision
     */
    public SITVersion(int major, int minor, int revision){
        this(major, minor, revision, null, null);
    }
    
    /**
     * Costruttore che accetta anche lo stato del rilascio e la relativa revisione. 
     * @param major
     * @param minor
     * @param revision
     * @param releaseStatus
     * @param releaseStatusRevision
     */
    public SITVersion(int major, int minor, int revision, ReleaseStatus releaseStatus, Integer releaseStatusRevision){
        this.major = major;
        this.minor = minor;
        this.revision = revision;
        this.releaseStatus = releaseStatus==null?ReleaseStatus.LIVE_RELEASE:releaseStatus;
        this.releaseStatusRevision = releaseStatusRevision;
    }

    /**
     * @see APIVersion#getMajor()
     */
    public int getMajor() {
        return this.major;
    }

    /**
     * @see APIVersion#getMinor()
     */
    public int getMinor() {
        return this.minor;
    }

    /**
     * Restituisce lo stato del rilascio di una versione. 
     * @return ReleaseStatus. Stato del rilascio
     */
    public ReleaseStatus getReleaseStatus() {
        return this.releaseStatus;
    }

    /**
     * @see Version#getRevision()
     */
    public int getRevision() {        
        return this.revision;
    }
    
    /**
     * Metodo statico che restituisce un oggetto SITVersion parsando una stringa che rappresenti la versione.
     * 
     * @param version
     * @return SITVersion
     * @throws SITParseException Lanciata qualora la stringa che rappresenta la versione non sia correttamente formattata.
     */
    public static SITVersion parse(String version) throws SITParseException {
        
        String regExp = "^(\\d+)\\.(\\d+)\\.(\\d+)-?((A|B|RC|SNAPSHOT)(\\d+)?)?$";
        Pattern pattern = Pattern.compile(regExp);    
        Matcher m = pattern.matcher(version);
        if(!m.matches()){
            throw new SITParseException("La versione fornita (" + version + ") non risulta avere un formato corretto : major.minor.revision[-releaseStatus[releaseStatusRevision]].");
        }
        
        String majorS = m.group(1);
        String minorS = m.group(2);
        String revisionS = m.group(3);
        String statusS = m.group(5);
        String statusRevisionS = m.group(6);
        
        int major = Integer.parseInt(majorS);
        int minor = Integer.parseInt(minorS);
        int revision = Integer.parseInt(revisionS);
        
        if(statusS == null){
            return new SITVersion(major,minor,revision);
        }
        ReleaseStatus status = ReleaseStatus.decode(statusS);
        if(status == null) {
            throw new SITParseException("AA Lo stato della release (" + statusS + ") non risulta essere fra quelli codificati.");
        }
        Integer statusRevision = (statusRevisionS == null)?null:Integer.parseInt(statusRevisionS);
        return new SITVersion(major,minor,revision,status,statusRevision);
        
    }

    /**
     * @see Version#format()
     */
    public String format() {
        MessageFormat mf = new MessageFormat("{0,number,#}.{1,number,#}.{2,number,#}{3}");
        String status = "";
        
        if(!getReleaseStatus().equals(ReleaseStatus.LIVE_RELEASE)){
            status += getReleaseStatus().equals(ReleaseStatus.SNAPSHOT)? "-" : "";        
            status += getReleaseStatus().getCodice();
            status += (getReleaseStatusRevision()!=null ? getReleaseStatusRevision().intValue() : "");             
        }
        
        Object[] arguments = {new Integer(getMajor()),new Integer(getMinor()),new Integer(getRevision()),status};        
        return mf.format(arguments);
    }

    /**
     * Restituisce la revisione di un determinato stato di rilascio.
     * <p>
     * Ad esempio : 
     * </p>
     * rilascio = A1 --> revisione del rilascio = 1 <br />
     * rilascio = B3 --> revisione del rilascio = 3 <br />
     * rilascio = RC2 --> revisione del rilascio = 2 
     * <br />
     * <br />
     * @return
     */
    public Integer getReleaseStatusRevision() {
        return this.releaseStatusRevision;
    }       
        
}
